﻿using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections.Generic;
using Microsoft.Build.Utilities;
using Microsoft.Build.Framework;
using Borland.Build.Tasks.Shared;

namespace RCCreateTask
{
    public class CreateRcFile : Task
    {
        public ITaskItem OutputFile { get; set; }
        public bool IncludeVerInfo { get; set; }
        public bool AutoGenVersion { get; set; }
        public int MajorVer { get; set; }
        public int MinorVer { get; set; }
        public int Release { get; set; }
        public int Build { get; set; }
        public bool Debug { get; set; }
        public bool PreRelease { get; set; }
        public bool Special { get; set; }
        public bool Private { get; set; }
        public bool DLL { get; set; }
        public int Locale { get; set; }
        public string KeysString { get; set; }
        public string ManifestFile { get; set; }            
        public string VCLCustomStyles { get; set; }
        public string Icon { get; set; }
        public string ResourcePlatform { get; set; }
        public string OSXOutputDir { get; set; }
        public int TargetedPlatforms { get; set; }
        public string ProjectName { get; set; }

        public override bool Execute()
        {
            if (IncludeVerInfo && OutputFile != null)
            {
                if (ResourcePlatform.Equals("Win32") || ResourcePlatform.Equals("Win64"))
                {
                    using (StreamWriter sw = new StreamWriter(OutputFile.ToString(), false, Encoding.Unicode))
                    {

                        sw.WriteLine("/* ----- VS_VERSION.dwFileFlags ----- */");
                        sw.WriteLine("#define VS_FF_DEBUG             0x00000001L");
                        sw.WriteLine("#define VS_FF_PRERELEASE        0x00000002L");
                        sw.WriteLine("#define VS_FF_PATCHED           0x00000004L");
                        sw.WriteLine("#define VS_FF_PRIVATEBUILD      0x00000008L");
                        sw.WriteLine("#define VS_FF_INFOINFERRED      0x00000010L");
                        sw.WriteLine("#define VS_FF_SPECIALBUILD      0x00000020L");
                        sw.WriteLine("");
                        sw.WriteLine("/* ----- VS_VERSION.dwFileOS ----- */");
                        sw.WriteLine("#define VOS_UNKNOWN             0x00000000L");
                        sw.WriteLine("#define VOS_DOS                 0x00010000L");
                        sw.WriteLine("#define VOS_OS216               0x00020000L");
                        sw.WriteLine("#define VOS_OS232               0x00030000L");
                        sw.WriteLine("#define VOS_NT                  0x00040000L");
                        sw.WriteLine("#define VOS_WINCE               0x00050000L");
                        sw.WriteLine("");
                        sw.WriteLine("#define VOS__BASE               0x00000000L");
                        sw.WriteLine("#define VOS__WINDOWS16          0x00000001L");
                        sw.WriteLine("#define VOS__PM16               0x00000002L");
                        sw.WriteLine("#define VOS__PM32               0x00000003L");
                        sw.WriteLine("#define VOS__WINDOWS32          0x00000004L");
                        sw.WriteLine("");
                        sw.WriteLine("#define VOS_DOS_WINDOWS16       0x00010001L");
                        sw.WriteLine("#define VOS_DOS_WINDOWS32       0x00010004L");
                        sw.WriteLine("#define VOS_OS216_PM16          0x00020002L");
                        sw.WriteLine("#define VOS_OS232_PM32          0x00030003L");
                        sw.WriteLine("#define VOS_NT_WINDOWS32        0x00040004L");
                        sw.WriteLine("");
                        sw.WriteLine("/* ----- VS_VERSION.dwFileType ----- */");
                        sw.WriteLine("#define VFT_UNKNOWN             0x00000000L");
                        sw.WriteLine("#define VFT_APP                 0x00000001L");
                        sw.WriteLine("#define VFT_DLL                 0x00000002L");
                        sw.WriteLine("#define VFT_DRV                 0x00000003L");
                        sw.WriteLine("#define VFT_FONT                0x00000004L");
                        sw.WriteLine("#define VFT_VXD                 0x00000005L");
                        sw.WriteLine("#define VFT_STATIC_LIB          0x00000007L");
                        sw.WriteLine("");

                        int LangId = Locale & 0xFFFF;
                        sw.WriteLine("LANGUAGE 0x{0:X2}, 0x{1:X2}", LangId & 0x3ff, LangId >> 10);

                        sw.WriteLine("");
                        Dictionary<string, string> VersionInfos = new Dictionary<string, string>();
                        if (KeysString != null)
                        {
                            string[] versions = KeysString.Split(';');
                            foreach (string item in versions)
                            {
                                string[] values = item.Split('=');
                                VersionInfos.Add(values[0], values[1]);
                            }
                        }

                        sw.WriteLine("1 VERSIONINFO LOADONCALL MOVEABLE DISCARDABLE IMPURE");
                        if ((MajorVer+MinorVer+Build+Release)>0)
                            sw.Write("FILEVERSION {0}, {1}, {2}, {3}", MajorVer, MinorVer, Release, Build);
                        else if (VersionInfos["FileVersion"] != null)
                        {
                            Regex re = new Regex("[^0-9]+");
                            string[] vernum = VersionInfos["FileVersion"].Split('.');
                            sw.Write("FILEVERSION {0}", vernum[0]);
                            for (int idx = 1; idx < vernum.Length; idx++)
                            {
                                int i;
                                string s = re.Replace(vernum[idx], "");
                                int.TryParse(s, out i);
                                sw.Write(", {0}", i);
                            }
                            sw.WriteLine("");
                        }
                        else
                            sw.WriteLine("FILEVERSION 0, 0, 0, 0");

                        if (Debug || Special || PreRelease || Private)
                        {
                            int flags = 0;
                            if (Debug)
                                flags |= 0x01;
                            if (Special)
                                flags |= 0x20;
                            if (PreRelease)
                                flags |= 0x02;
                            if (Private)
                                flags |= 0x08;
                            sw.Write("FILEFLAGS {0:X2}", flags);
                        }
                        else
                            sw.WriteLine("FILEFLAGS 0x0L");

                        if ((MajorVer + MinorVer + Build + Release) > 0)
                            sw.Write("PRODUCTVERSION {0}, {1}, {2}, {3}", MajorVer, MinorVer, Release, Build);
                        else if (VersionInfos["ProductVersion"] != null)
                        {
                            Regex re = new Regex("[^0-9]+");
                            string[] vernum = VersionInfos["ProductVersion"].Split('.');
                            sw.Write("PRODUCTVERSION {0}", vernum[0]);
                            for (int idx = 1; idx < vernum.Length; idx++)
                            {
                                int i;
                                string s = re.Replace(vernum[idx], "");
                                int.TryParse(s, out i);
                                sw.Write(", {0}", i);
                            }
                            sw.WriteLine("");
                        }
                        else
                            sw.WriteLine("PRODUCTVERSION 0, 0, 0, 0");

                        sw.WriteLine("FILEOS VOS__WINDOWS32");
                        if (DLL)
                            sw.WriteLine("FILETYPE VFT_DLL");
                        else
                            sw.WriteLine("FILETYPE VFT_APP");
                        sw.WriteLine("{");
                        sw.WriteLine(" BLOCK \"StringFileInfo\"");
                        sw.WriteLine(" {");
                        sw.WriteLine("  BLOCK \"{0:X4}{1:X4}\"", Locale, 1200);
                        sw.WriteLine("  {");
                        foreach (KeyValuePair<string, string> item in VersionInfos)
                        {
                            if (item.Value.Length > 0)
                                sw.WriteLine("   VALUE \"" + item.Key + "\",  \"" + item.Value + "\\0\"");
                        }
                        sw.WriteLine("  }");
                        sw.WriteLine(" }");
                        sw.WriteLine(" BLOCK \"VarFileInfo\"");
                        sw.WriteLine(" {");
                        sw.WriteLine("  VALUE \"Translation\", {0:D}, 1200", Locale);
                        sw.WriteLine(" }");
                        sw.WriteLine("}");
                        sw.WriteLine("");
                        if (Icon != null)
                            sw.WriteLine("MAINICON ICON \"" + Icon.Replace("\\", "\\\\") + "\"");
                        if (ManifestFile != null)
                            sw.WriteLine("1 24 \"" + ManifestFile.Replace("\\", "\\\\") + "\"");

                        if (VCLCustomStyles != null)
                        {
                            string[] Styles = VCLCustomStyles.Split(';');
                            sw.WriteLine("PLATFORMTARGETS RCDATA {{{0}}}", Styles.Length + 1);
                            foreach (string item in Styles)
                            {
                                string[] vals = item.Trim('\"').Split('|');
                                sw.WriteLine("{0} {1} \"{2}\"", vals[0].Replace(" ", ""), vals[1], vals[2].Replace("\\", "\\\\"));
                            }
                        }
                        else
                            sw.WriteLine("PLATFORMTARGETS RCDATA {1}");
                        sw.Close();
                    }
                }
                else
                {
                    Borland.Build.Tasks.Shared.CreateRcFile rc = new Borland.Build.Tasks.Shared.CreateRcFile();
                    rc.AutoGenVersion = AutoGenVersion;
                    rc.Debug = Debug;
                    rc.Icon = Icon;
                    rc.OutputFile = OutputFile;
                    rc.IncludeVerInfo = IncludeVerInfo;
                    rc.MajorVer = MajorVer;
                    rc.MinorVer = MajorVer;
                    rc.Release = Release;
                    rc.Build = Build;
                    rc.PreRelease = PreRelease;
                    rc.Special = Special;
                    rc.Private = Private;
                    rc.DLL = DLL;
                    rc.Locale = Locale;
                    rc.KeysString = KeysString;
                    rc.ManifestFile = ManifestFile;
                    rc.VCLCustomStyles = VCLCustomStyles;
                    rc.ResourcePlatform = ResourcePlatform;
                    rc.OSXOutputDir = OSXOutputDir;
                    rc.TargetedPlatforms = TargetedPlatforms;
                    rc.ProjectName = ProjectName;
                    rc.Execute();
                }
            }
            return true;
        }
    }
}
